/* eslint-disable no-magic-numbers */
import getPolygonCoords from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/coords/getPolygonCoords';
import getEllipseCoords from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/coords/getEllipseCoords';
import { UsePointToProjectionResult } from '@/utils/map/usePointToProjection';
import Polygon from 'ol/geom/Polygon';
import { Coordinate } from 'ol/coordinate';
import { Shape } from '@/types/models';
import getCentroid from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/coords/getCentroid';

export default function getMultiPolygon({
  x,
  y,
  width,
  height,
  shapes,
  pointToProjection,
  mirror,
}: {
  x: number;
  y: number;
  width: number;
  height: number;
  shapes: Array<Shape>;
  pointToProjection: UsePointToProjectionResult;
  mirror?: boolean;
}): Array<Polygon> {
  return shapes.map(shape => {
    let coords: Array<Coordinate> = [];
    if (shape.type === 'POLYGON') {
      coords = getPolygonCoords({ points: shape.points, x, y, height, width, pointToProjection });
    } else if (shape.type === 'ELLIPSE') {
      coords = getEllipseCoords({
        x,
        y,
        center: shape.center,
        radius: shape.radius,
        height,
        width,
        pointToProjection,
      });
    }

    if (mirror) {
      const centroid = getCentroid(coords);

      coords = coords.map(coord => {
        const mirroredX = 2 * centroid[0] - coord[0];

        return [mirroredX, coord[1]];
      });
    }

    return new Polygon([coords]);
  });
}
